@@ -4,8 +4,20 @@
  *
  * Copyright 1997,1998 Alexandre Oliva <oliva@dcc.unicamp.br>
  *
- * See the files "COPYING" and "README" for information on usage and
- * redistribution of this file.  */
+ * This file is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 
 #include "guarana/GuaraNative.h"
 
@@ -34,6 +46,7 @@ NativeOperation *guarana_new_native_operation(struct Hjava_lang_Object *object,
   op->object = object;
   op->thread = thread;
   op->replaced = replaced;
+  op->kind = replaced ? op_performed : 0;
   return op;
 }
 
@@ -187,9 +200,9 @@ guarana_slots_reify_method_invocation(struct Hjava_lang_Object *obj, Method *met
     op = guarana_new_native_operation(obj, getCurrentThread(), 0);
     if (equalUtf8Consts(method->name, init_name) ||
 	equalUtf8Consts(method->name, constructor_name))
-      op->kind = op_constructor_invocation;
+      op->kind |= op_constructor_invocation;
     else
-      op->kind = op_method_invocation;
+      op->kind |= op_method_invocation;
     op->op_id.method = method;
     sig = op->op_id.method->signature->data;
     argcount = sizeofSig(&sig, true);
@@ -376,9 +389,9 @@ guarana_va_list_reify_method_invocation(struct HBR_unicamp_Guarana_MetaObject *m
   op = guarana_new_native_operation(obj, getCurrentThread(), 0);
   if (equalUtf8Consts(method->name, init_name) ||
       equalUtf8Consts(method->name, constructor_name))
-    op->kind = op_constructor_invocation;
+    op->kind |= op_constructor_invocation;
   else
-    op->kind = op_method_invocation;
+    op->kind |= op_method_invocation;
   op->op_id.method = method;
   sig = method->signature->data;
   argcount = sizeofSig(&sig, true);
@@ -453,13 +466,20 @@ guarana_va_list_reify_method_invocation(struct HBR_unicamp_Guarana_MetaObject *m
   }
 }
 
+void
+guarana_va_list_resolve_method_invocation(struct HBR_unicamp_Guarana_MetaObject *mo, struct Hjava_lang_Object *obj, Method *method, va_list args, slots *retval) {
+  if (!METHOD_IS_STATIC(method))
+      method = DMETHOD_DESC(obj->dtable->method[method->idx]);
+  guarana_va_list_reify_method_invocation(mo, obj, method, args, retval);
+}
+
 void
 guarana_reify_field_load(struct HBR_unicamp_Guarana_MetaObject *mo, struct Hjava_lang_Object *o, Hjava_lang_Class *clazz, Field *field, jvalue *wrap) {
   NativeOperation *op;
   NativeResult *res;
 
   op = guarana_new_native_operation(o, getCurrentThread(), 0);
-  op->kind = op_field_read;
+  op->kind |= op_field_read;
   op->op_id.field_id.clazz = clazz;
   op->op_id.field_id.field = field;
 
@@ -536,7 +556,7 @@ guarana_reify_field_store(struct HBR_unicamp_Guarana_MetaObject *mo, struct Hjav
   NativeOperation *op;
 
   op = guarana_new_native_operation(o, getCurrentThread(), 0);
-  op->kind = op_field_write;
+  op->kind |= op_field_write;
   op->op_id.field_id.clazz = clazz;
   op->op_id.field_id.field = field;
   op->op_arg = *val;
@@ -613,7 +633,7 @@ guarana_reify_array_load(struct HBR_unicamp_Guarana_MetaObject *mo, struct Hjava
   NativeResult *res;
 
   op = guarana_new_native_operation(o, getCurrentThread(), 0);
-  op->kind = op_array_read;
+  op->kind |= op_array_read;
   op->op_id.index = index;
 
   res = guarana_handle_operation(op);
@@ -689,7 +709,7 @@ guarana_reify_array_store(struct HBR_unicamp_Guarana_MetaObject *mo, struct Hjav
   NativeOperation *op;
 
   op = guarana_new_native_operation(o, getCurrentThread(), 0);
-  op->kind = op_array_write;
+  op->kind |= op_array_write;
   op->op_id.index = index;
   op->op_arg = *val;
 
@@ -759,7 +779,7 @@ jdouble_reify_array_store(struct HBR_unicamp_Guarana_MetaObject *mo, struct Hjav
   guarana_reify_array_store(mo, o, index, &wrap);
 }
 
-int
+jint
 guarana_reify_arraylength(struct Hjava_lang_Object *o) {
   struct HBR_unicamp_Guarana_MetaObject *mo = o->meta_object;
   NativeOperation *op;
@@ -770,7 +790,7 @@ guarana_reify_arraylength(struct Hjava_lang_Object *o) {
     return obj_length((struct Array*)o);
 
   op = guarana_new_native_operation(o, getCurrentThread(), 0);
-  op->kind = op_array_length;
+  op->kind |= op_array_length;
 
   res = guarana_handle_operation(op);
 
@@ -783,7 +803,7 @@ void
 guarana_reify_monitorEnter(struct Hjava_lang_Object *o) {
   if (o->meta_object) {
     NativeOperation *op = guarana_new_native_operation(o, getCurrentThread(), 0);
-    op->kind = op_monitor_enter;
+    op->kind |= op_monitor_enter;
 
     guarana_handle_operation(op);
   } else
@@ -794,7 +814,7 @@ void
 guarana_reify_monitorExit(struct Hjava_lang_Object *o) {
   if (o->meta_object) {
     NativeOperation *op = guarana_new_native_operation(o, getCurrentThread(), 0);
-    op->kind = op_monitor_exit;
+    op->kind |= op_monitor_exit;
 
     guarana_handle_operation(op);
   } else
